import os
import unittest
from pathlib import Path
from tests.helpers import delete_files_in_folder


class TestHelpers(unittest.TestCase):
    def setUp(self):
        self.temp_dir = "tests/tmp"

    def test_delete_files_in_folder(self):
        "test helpers function to clean out tests temp directory"
        file_name = os.path.join(self.temp_dir, "test_file.txt")
        folder_name = os.path.join(self.temp_dir, "test_folder")
        os.mkdir(folder_name)
        Path(file_name).touch()
        delete_files_in_folder(self.temp_dir, filter_out=[".keepme"])
        self.assertEqual(len(os.listdir(self.temp_dir)), 1)



import re
from xml.etree import ElementTree
from xml.etree.ElementTree import Element, SubElement
from elifecleaner import LOGGER, utils


FIG_LABEL_CONTENT_PATTERN = r"[A-Za-z ]+ image [0-9]+?\.{0,1}"


def match_fig_label_content(content):
    "check if the content is a label for a figure"
    return bool(re.match(FIG_LABEL_CONTENT_PATTERN, content)) if content else False


TABLE_LABEL_CONTENT_PATTERN = r"[A-Za-z ]+ table [0-9]+?\.{0,1}"


def match_table_label_content(content):
    "check if the content is a label for a table"
    return bool(re.match(TABLE_LABEL_CONTENT_PATTERN, content)) if content else False


def is_p_label(tag, sub_article_id, p_tag_index, block_type, identifier):
    "check if the p tag contents is a label"
    label = None
    if tag.find("bold") is not None:
        label = tag.find("bold").text
        LOGGER.info(
            '%s potential label "%s" in p tag %s of id %s',
            identifier,
            label,
            p_tag_index,
            sub_article_id,
        )
    if block_type == "fig":
        return match_fig_label_content(label)
    elif block_type == "table":
        return match_table_label_content(label)
    return None


def is_p_inline_graphic(tag, sub_article_id, p_tag_index, identifier):
    "check if the p tag contains only an inline-graphic tag"
    # simple check for a tag first
    if tag.find("inline-graphic") is None:
        LOGGER.info(
            "%s no inline-graphic tag found in p tag %s of id %s",
            identifier,
            p_tag_index,
            sub_article_id,
        )
        return False
    # check if the p tag contains an inline-grpahic only, ignoring whitespace
    if (
        tag.find("inline-graphic") is not None
        and (not tag.text or not tag.text.rstrip())
        and (
            not tag.find("inline-graphic").tail
            or not tag.find("inline-graphic").tail.rstrip()
        )
    ):
        LOGGER.info(
            "%s only inline-graphic tag found in p tag %s of id %s",
            identifier,
            p_tag_index,
            sub_article_id,
        )
        return True
    # default return None
    return None


def sub_article_tag_parts(sub_article_root):
    "return the id and body tag from a sub-article tag"
    sub_article_id = sub_article_root.get("id")
    body_tag = sub_article_root.find("body")
    return sub_article_id, body_tag


def tag_index_groups(body_tag, sub_article_id, block_type, identifier):
    "iterate through the tags in body_tag and find groups of tags to be converted to a fig"
    fig_index_groups = []
    if not body_tag:
        return fig_index_groups
    label_index = None
    caption_index = None
    for tag_index, child_tag in enumerate(body_tag.iterfind("*")):
        if child_tag.tag == "p":
            if label_index is None:
                # match figure label
                if is_p_label(
                    child_tag, sub_article_id, tag_index, block_type, identifier
                ):
                    label_index = tag_index
                    LOGGER.info(
                        "%s label p tag index %s of id %s",
                        identifier,
                        label_index,
                        sub_article_id,
                    )
            elif label_index is not None and caption_index is None:
                # look for optional caption
                if not is_p_inline_graphic(
                    child_tag, sub_article_id, tag_index, identifier
                ):
                    caption_index = tag_index
                    # loop to the next p tag
                    continue
                # if no caption found, check this p tag for an inline-graphic below
                caption_index = False
            if label_index is not None and caption_index is not None:
                # look for inline graphic to be converted to fig
                if is_p_inline_graphic(
                    child_tag, sub_article_id, tag_index, identifier
                ):
                    # add to the fig index group
                    fig_p_data = {
                        "label_index": label_index,
                        "caption_index": caption_index,
                        "inline_graphic_index": tag_index,
                    }
                    fig_index_groups.append(fig_p_data)
                # reset the indexes
                label_index = None
                caption_index = None
        else:
            # if is not a p tag, reset the indexes
            label_index = None
            caption_index = None
    return fig_index_groups


def graphic_href_list(body_tag, index_groups):
    "collect a list of xlink:href values of graphic tags from the index_groups"
    href_list = []
    for group in index_groups:
        if group.get("inline_graphic_index") is not None:
            inline_graphic_p = body_tag[group.get("inline_graphic_index")]
            inline_graphic_tag = inline_graphic_tag_from_tag(inline_graphic_p)
            image_href = utils.xlink_href(inline_graphic_tag)
            if image_href:
                href_list.append(image_href)
    return href_list


def title_paragraph_content(string_list):
    "from list of strings repair inline formatting tags and split into title and paragraph"
    # check for nested inline formatting tags
    title_content = ""
    content_remainders = []
    for title_part in string_list:
        if not title_content:
            title_content += title_part
            continue
        open_tag_count = title_content.count(utils.open_tag("italic"))
        close_tag_count = title_content.count(utils.close_tag("italic"))
        open_bold_tag_count = title_content.count(utils.open_tag("bold"))
        close_bold_tag_count = title_content.count(utils.close_tag("bold"))
        open_ext_link_tag_count = title_content.count(
            utils.open_tag("ext-link").rstrip(">")
        )
        close_ext_link_tag_count = title_content.count(utils.close_tag("ext-link"))
        if (
            open_tag_count != close_tag_count
            or open_bold_tag_count != close_bold_tag_count
            or open_ext_link_tag_count != close_ext_link_tag_count
        ):
            title_content += title_part
        else:
            content_remainders.append(title_part)
    title_content = title_content.lstrip()
    paragraph_content = None
    if content_remainders:
        paragraph_content = "".join(content_remainders)
    return title_content, paragraph_content


def set_graphic_tag(parent, image_href, new_file_name):
    "add a graphic tag to the parent"
    graphic_tag = SubElement(parent, "graphic")
    graphic_tag.set("mimetype", "image")
    graphic_tag.set("mime-subtype", utils.file_extension(image_href))
    graphic_tag.set("{http://www.w3.org/1999/xlink}href", new_file_name)


def caption_title_paragraph(tag):
    "split the content into title and optional paragraph for a fig caption"
    xml_string = ElementTree.tostring(tag)
    if isinstance(xml_string, bytes):
        xml_string = str(xml_string, encoding="utf-8")

    # split the string into parts
    string_list = split_title_parts(xml_string)

    title_content, paragraph_content = title_paragraph_content(string_list)

    # fix enclosing tags
    if paragraph_content:
        title_content = "%s</p>" % title_content
        paragraph_content = "<p %s>%s" % (
            utils.namespace_string(),
            paragraph_content.lstrip(),
        )
    # parse XML string back into an Element
    caption_title_p_tag = ElementTree.fromstring(title_content)
    caption_paragraph_p_tag = None
    if paragraph_content:
        caption_paragraph_p_tag = ElementTree.fromstring(paragraph_content)

    return caption_title_p_tag, caption_paragraph_p_tag


def set_caption_tag(parent, body_tag, caption_index):
    "add a caption tag to the parent"
    caption_tag = SubElement(parent, "caption")

    # split into title and p tag portions
    caption_title_p_tag, caption_paragraph_p_tag = caption_title_paragraph(
        body_tag[caption_index]
    )

    caption_title_tag = SubElement(caption_tag, "title")
    caption_title_tag.text = caption_title_p_tag.text
    # handle if there are child tags in the title
    for child_tag in caption_title_p_tag.iterfind("*"):
        caption_title_tag.append(child_tag)
        caption_title_tag.tail = caption_title_p_tag.tail
    if caption_paragraph_p_tag is not None and caption_paragraph_p_tag.text:
        caption_p_tag = SubElement(caption_tag, "p")
        caption_p_tag.text = caption_paragraph_p_tag.text
        # handle if there are child tags in the caption
        for child_tag in caption_paragraph_p_tag.iterfind("*"):
            caption_p_tag.append(child_tag)
            caption_p_tag.tail = caption_paragraph_p_tag.tail


def split_title_parts(xml_string):
    "split the XML string into parts to be processed by caption_title_paragraph()"
    title_parts = []
    tag_match_pattern = re.compile(r"(<.*?>)")
    match_tag_groups = tag_match_pattern.split(xml_string)
    string_part = ""
    for tag_group in match_tag_groups:
        if "." in tag_group and "<" not in tag_group:
            if tag_group == ".":
                dot_parts = ["."]
            else:
                dot_parts = tag_group.split(".")

            for dot_index, dot_part in enumerate(dot_parts):
                suffix = ""
                if dot_index + 1 < len(dot_parts):
                    suffix = "."
                string_part += "%s%s" % (dot_part, suffix)
                title_parts.append(string_part)
                string_part = ""
        else:
            string_part += tag_group
    # append the final content
    if title_parts:
        title_parts[-1] += string_part
    elif string_part:
        # only one string found
        title_parts.append(string_part)

    return title_parts


def strip_tag_text(tag):
    "remove whitespace to the left of the child tag if present"
    if isinstance(tag, Element) and tag.text is not None and not tag.text.rstrip():
        tag.text = None


def strip_tag_tail(tag):
    "remove whitespace to the right of the tag if present"
    if isinstance(tag, Element) and tag.tail is not None and not tag.tail.rstrip():
        tag.tail = None


def inline_graphic_tag_from_tag(tag):
    "get the inline-graphic tag from the parent tag"
    strip_tag_text(tag)
    inline_graphic_tag = tag.find("inline-graphic")
    strip_tag_tail(inline_graphic_tag)
    return inline_graphic_tag


def set_label_tag(parent, body_tag, label_index):
    "add a label tag to the parent"
    # insert tags into original inline-graphic
    label_tag = SubElement(parent, "label")
    # copy content from the label_index p tag
    label_tag.text = body_tag[label_index].find("bold").text



import csv
from io import StringIO
from xml.etree.ElementTree import Element, SubElement
from jatsgenerator.utils import append_to_tag
from elifecleaner import block, utils


def table_wrap_id(sub_article_id, table_index):
    "create an id attribute for a table-wrap tag"
    return "%stable%s" % (sub_article_id, table_index)


def table_file_name(inf_file_name, sub_article_id, table_index):
    "from inf file name create a new table file name"
    return inf_file_name.replace(
        utils.inf_file_identifier(inf_file_name),
        "%s-table%s" % (sub_article_id, table_index),
    )


def table_tag_index_groups(body_tag, sub_article_id, identifier):
    "iterate through the tags in body_tag and find groups of tags to be converted to a table-wrap"
    return block.tag_index_groups(body_tag, sub_article_id, "table", identifier)


def table_inline_graphic_hrefs(sub_article_root, identifier):
    "get inline-graphic href values"
    sub_article_id, body_tag = block.sub_article_tag_parts(sub_article_root)
    href_list = []
    if body_tag is not None:
        # match paragraphs with table data in them and record the tag indexes
        table_index_groups = table_tag_index_groups(
            body_tag, sub_article_id, identifier
        )
        href_list = block.graphic_href_list(body_tag, table_index_groups)
    return href_list


def table_graphic_hrefs(sub_article_root, identifier):
    "get table-wrap graphic href values"
    sub_article_id, body_tag = block.sub_article_tag_parts(sub_article_root)
    href_list = []
    if body_tag is not None:
        for graphic_tag in body_tag.findall(".//table-wrap/graphic"):
            image_href = utils.xlink_href(graphic_tag)
            if image_href:
                href_list.append(image_href)
    return href_list


def transform_table_group(body_tag, table_index, table_group, sub_article_id):
    "transform one set of p tags into table-wrap tags as specified in the table_group dict"
    inline_graphic_p_tag = body_tag[table_group.get("inline_graphic_index")]
    inline_graphic_tag = block.inline_graphic_tag_from_tag(inline_graphic_p_tag)
    image_href = utils.xlink_href(inline_graphic_tag)

    # insert tags into original inline-graphic
    block.set_label_tag(inline_graphic_p_tag, body_tag, table_group.get("label_index"))

    # caption
    if table_group.get("caption_index"):
        block.set_caption_tag(
            inline_graphic_p_tag, body_tag, table_group.get("caption_index")
        )

    # rename the image file
    new_file_name = table_file_name(image_href, sub_article_id, table_index)

    # graphic tag
    block.set_graphic_tag(inline_graphic_p_tag, image_href, new_file_name)

    # convert inline-graphic p tag to a table-wrap tag
    inline_graphic_p_tag.tag = "table-wrap"
    inline_graphic_p_tag.set("id", table_wrap_id(sub_article_id, table_index))

    # delete the old inline-graphic tag
    inline_graphic_p_tag.remove(inline_graphic_tag)

    # remove the old p tags
    if table_group.get("caption_index"):
        del body_tag[table_group.get("caption_index")]
    del body_tag[table_group.get("label_index")]


def transform_table_groups(body_tag, table_index_groups, sub_article_id):
    "transform p tags in the body_tag to table-wrap tags as listed in table_index_groups"
    # transform the table tags in reverse order
    table_index = len(table_index_groups)
    for table_group in reversed(table_index_groups):
        transform_table_group(body_tag, table_index, table_group, sub_article_id)
        # decrement the index
        table_index -= 1


def transform_table(sub_article_root, identifier):
    "transform inline-graphic tags and related p tags into a table-wrap tag"
    sub_article_id, body_tag = block.sub_article_tag_parts(sub_article_root)
    if body_tag is not None:
        # match paragraphs with data in them and record the tag indexes
        table_index_groups = table_tag_index_groups(
            body_tag, sub_article_id, identifier
        )
        transform_table_groups(body_tag, table_index_groups, sub_article_id)
    return sub_article_root


def tsv_to_list(tsv_string):
    "convert Tab Separated Value (TSV) string to a list"
    table_rows = []
    reader = csv.reader(StringIO(tsv_string), delimiter="\t")
    for row in reader:
        table_rows.append(row)
    return table_rows


def list_to_table_xml(table_rows):
    "convert list of rows into table XML"
    table_tag = Element("table")
    for row_index, row in enumerate(table_rows):
        if row_index <= 0:
            block_tag = SubElement(table_tag, "thead")
            cell_tag_name = "th"
        elif row_index == 1:
            block_tag = SubElement(table_tag, "tbody")
            cell_tag_name = "td"
        tr_tag = SubElement(block_tag, "tr")
        for cell in row:
            # replace new line charactes with a break tag
            cell = cell.replace("\n", "<break/>")
            # strip whitespace
            cell = cell.lstrip().rstrip()
            # append to the tr tag
            append_to_tag(tr_tag, cell_tag_name, cell)
    return table_tag



import os
import unittest
import zipfile
from collections import OrderedDict
from xml.etree import ElementTree
from mock import patch
import wand
from elifecleaner import LOGGER, configure_logging, parse, pdf_utils, zip_lib
from elifecleaner.utils import CONTROL_CHARACTER_ENTITY_REPLACEMENT
from tests.helpers import delete_files_in_folder, read_fixture, read_log_file_lines


class TestParse(unittest.TestCase):
    def test_file_extension(self):
        self.assertEqual(parse.file_extension("image.JPG"), "jpg")
        self.assertEqual(parse.file_extension("folder/figure.pdf"), "pdf")
        self.assertEqual(parse.file_extension("test"), None)
        self.assertIsNone(parse.file_extension(None))

    def test_pdf_page_count_blank(self):
        self.assertIsNone(parse.pdf_page_count(""))


class TestCheckEjpZip(unittest.TestCase):
    def setUp(self):
        self.temp_dir = "tests/tmp"
        self.log_file = os.path.join(self.temp_dir, "test.log")
        self.log_handler = configure_logging(self.log_file)
        self.original_repair_xml_value = parse.REPAIR_XML

    def tearDown(self):
        LOGGER.removeHandler(self.log_handler)
        delete_files_in_folder(self.temp_dir, filter_out=[".keepme"])
        parse.REPAIR_XML = self.original_repair_xml_value

    @patch.object(parse, "check_art_file")
    @patch.object(pdf_utils, "pdf_image_pages")
    @patch.object(pdf_utils, "pdfimages_exists")
    def test_check_ejp_zip(
        self, mock_pdfimages_exists, mock_pdf_image_pages, fake_check_art_file
    ):
        fake_check_art_file.return_value = True
        mock_pdfimages_exists.return_value = True
        mock_pdf_image_pages.return_value = {1, 2}
        zip_file = "tests/test_data/30-01-2019-RA-eLife-45644.zip"
        zip_file_name = zip_file.split(os.sep)[-1]
        log_prefix = (
            "elifecleaner:parse:check_multi_page_figure_pdf: %s"
        ) % zip_file_name
        warning_prefix = ("WARNING %s multiple page PDF figure file:") % log_prefix
        info_prefix = ("INFO %s using pdfimages to check PDF figure file:") % log_prefix
        info_pages_prefix = (
            "INFO %s pdfimages found images on pages {1, 2} in PDF figure file:"
        ) % log_prefix
        expected = [
            "%s 30-01-2019-RA-eLife-45644/Appendix 1figure 10.pdf\n" % info_prefix,
            "%s 30-01-2019-RA-eLife-45644/Appendix 1figure 10.pdf\n"
            % info_pages_prefix,
            "%s 30-01-2019-RA-eLife-45644/Appendix 1figure 10.pdf\n" % warning_prefix,
            "%s 30-01-2019-RA-eLife-45644/Appendix 1figure 11.pdf\n" % info_prefix,
            "%s 30-01-2019-RA-eLife-45644/Appendix 1figure 11.pdf\n"
            % info_pages_prefix,
            "%s 30-01-2019-RA-eLife-45644/Appendix 1figure 11.pdf\n" % warning_prefix,
        ]
        result = parse.check_ejp_zip(zip_file, self.temp_dir)
        self.assertTrue(result)
        self.assertEqual(read_log_file_lines(self.log_file), expected)

    def test_check_ejp_zip_do_not_repair_xml(self):
        parse.REPAIR_XML = False
        zip_file = "tests/test_data/30-01-2019-RA-eLife-45644.zip"
        with self.assertRaises(ElementTree.ParseError):
            parse.check_ejp_zip(zip_file, self.temp_dir)
        log_file_lines = read_log_file_lines(self.log_file)
        self.assertTrue(log_file_lines[0].startswith("ERROR"))

    @patch.object(parse, "check_art_file")
    def test_check_ejp_zip_missing_file(self, fake_check_art_file):
        fake_check_art_file.return_value = True
        zip_file = "tests/test_data/08-11-2020-FA-eLife-64719.zip"
        # remove a file from a copy of the zip file for testing
        test_zip_file_name = os.path.join(self.temp_dir, "test_missing_file.zip")
        zip_file_name = test_zip_file_name.split(os.sep)[-1]

        remove_files = ["08-11-2020-FA-eLife-64719/eLife64719_figure2_classB.png"]
        with zipfile.ZipFile(zip_file, "r") as input_zipfile:
            with zipfile.ZipFile(test_zip_file_name, "w") as output_zipfile:
                for zip_info in input_zipfile.infolist():
                    if zip_info.filename not in remove_files:
                        output_zipfile.writestr(
                            zip_info, input_zipfile.read(zip_info.filename)
                        )

        warning_prefix = (
            "WARNING elifecleaner:parse:check_missing_files: %s" % zip_file_name
        )
        missing_file_prefix = "does not contain a file in the manifest:"
        expected = [
            "%s %s eLife64719_figure2_classB.png\n"
            % (warning_prefix, missing_file_prefix),
        ]

        result = parse.check_ejp_zip(test_zip_file_name, self.temp_dir)
        self.assertTrue(result)
        self.assertEqual(read_log_file_lines(self.log_file), expected)

    @patch.object(parse, "check_art_file")
    def test_check_ejp_zip_extra_file(self, fake_check_art_file):
        fake_check_art_file.return_value = True
        zip_file = "tests/test_data/08-11-2020-FA-eLife-64719.zip"
        # alter the manifest XML in the zip file for testing
        test_zip_file_name = os.path.join(self.temp_dir, "test_missing_file.zip")
        zip_file_name = test_zip_file_name.split(os.sep)[-1]

        xml_file_name = "08-11-2020-FA-eLife-64719/08-11-2020-FA-eLife-64719.xml"

        with zipfile.ZipFile(zip_file, "r") as input_zipfile:
            with zipfile.ZipFile(test_zip_file_name, "w") as output_zipfile:
                for zip_info in input_zipfile.infolist():
                    if zip_info.filename == xml_file_name:
                        # replace <file> tags with <file_dummy> tags in the manifest XML
                        mainfest_xml = input_zipfile.read(zip_info.filename)
                        mainfest_xml = mainfest_xml.replace(
                            b"<file", b"<file_dummy"
                        ).replace(b"</file", b"</file_dummy")
                        output_zipfile.writestr(zip_info, mainfest_xml)
                    else:
                        output_zipfile.writestr(
                            zip_info, input_zipfile.read(zip_info.filename)
                        )

        warning_prefix = (
            "WARNING elifecleaner:parse:check_extra_files: %s" % zip_file_name
        )
        extra_file_prefix = "has file not listed in the manifest:"
        expected = [
            "%s %s 08-11-2020-FA-eLife-64719.pdf\n"
            % (warning_prefix, extra_file_prefix),
            "%s %s eLife64719_figure1_classB.png\n"
            % (warning_prefix, extra_file_prefix),
            "%s %s eLife64719_figure2_classB.png\n"
            % (warning_prefix, extra_file_prefix),
            "%s %s eLife64719_template5.docx\n" % (warning_prefix, extra_file_prefix),
        ]

        result = parse.check_ejp_zip(test_zip_file_name, self.temp_dir)
        self.assertTrue(result)
        self.assertEqual(read_log_file_lines(self.log_file), expected)

    @patch.object(parse, "check_art_file")
    def test_check_ejp_zip_missing_file_by_name(self, fake_check_art_file):
        fake_check_art_file.return_value = True
        zip_file = "tests/test_data/08-11-2020-FA-eLife-64719.zip"
        # alter the manifest XML in the zip file for testing
        test_zip_file_name = os.path.join(self.temp_dir, "test_missing_file.zip")
        zip_file_name = test_zip_file_name.split(os.sep)[-1]

        xml_file_name = "08-11-2020-FA-eLife-64719/08-11-2020-FA-eLife-64719.xml"

        with zipfile.ZipFile(zip_file, "r") as input_zipfile:
            with zipfile.ZipFile(test_zip_file_name, "w") as output_zipfile:
                for zip_info in input_zipfile.infolist():
                    if zip_info.filename == xml_file_name:
                        # replace <file> tags with <file_dummy> tags in the manifest XML
                        mainfest_xml = input_zipfile.read(zip_info.filename)
                        mainfest_xml = mainfest_xml.replace(
                            b"<meta-value>Figure 2</meta-value>",
                            b"<meta-value>Figure 3</meta-value>",
                        )
                        output_zipfile.writestr(zip_info, mainfest_xml)
                    else:
                        output_zipfile.writestr(
                            zip_info, input_zipfile.read(zip_info.filename)
                        )

        warning_prefix = (
            "WARNING elifecleaner:parse:check_missing_files_by_name: %s" % zip_file_name
        )
        extra_file_prefix = "has file missing from expected numeric sequence:"
        expected = [
            "%s %s Figure 2\n" % (warning_prefix, extra_file_prefix),
        ]

        result = parse.check_ejp_zip(test_zip_file_name, self.temp_dir)
        self.assertTrue(result)
        self.assertEqual(read_log_file_lines(self.log_file), expected)


class TestArticleXmlAsset(unittest.TestCase):
    def setUp(self):
        self.temp_dir = "tests/tmp"

    def tearDown(self):
        delete_files_in_folder(self.temp_dir, filter_out=[".keepme"])

    def test_article_xml_name(self):
        zip_file = "tests/test_data/30-01-2019-RA-eLife-45644.zip"
        asset_file_name_map = zip_lib.unzip_zip(zip_file, self.temp_dir)
        expected = (
            "30-01-2019-RA-eLife-45644/30-01-2019-RA-eLife-45644.xml",
            "tests/tmp/30-01-2019-RA-eLife-45644/30-01-2019-RA-eLife-45644.xml",
        )
        xml_asset = parse.article_xml_asset(asset_file_name_map)
        self.assertEqual(xml_asset, expected)

    def test_article_xml_name_empty(self):
        asset_file_name_map = []
        expected = None
        xml_asset = parse.article_xml_asset(asset_file_name_map)
        self.assertEqual(xml_asset, expected)


class TestPdfPageCount(unittest.TestCase):
    def setUp(self):
        self.temp_dir = "tests/tmp"
        self.log_file = os.path.join(self.temp_dir, "test.log")
        self.log_handler = configure_logging(self.log_file)

    def tearDown(self):
        LOGGER.removeHandler(self.log_handler)
        delete_files_in_folder(self.temp_dir, filter_out=[".keepme"])

    @patch.object(wand.image.Image, "allocate")
    def test_pdf_page_count_wand_runtime_error(self, mock_image_allocate):
        mock_image_allocate.side_effect = wand.exceptions.WandRuntimeError()
        zip_lib.unzip_zip(
            "tests/test_data/30-01-2019-RA-eLife-45644.zip", self.temp_dir
        )
        pdf_path = "tests/tmp/30-01-2019-RA-eLife-45644/Appendix 1figure 10.pdf"
        with self.assertRaises(wand.exceptions.WandRuntimeError):
            self.assertIsNone(parse.pdf_page_count(pdf_path))
        expected = (
            "ERROR elifecleaner:parse:pdf_page_count: "
            "WandRuntimeError in pdf_page_count(), imagemagick may not be installed\n"
        )
        log_file_lines = read_log_file_lines(self.log_file)
        self.assertEqual(log_file_lines[0], expected)

    @patch.object(wand.image.Image, "allocate")
    def test_pdf_page_count_wand_policy_error(self, mock_image_allocate):
        mock_image_allocate.side_effect = wand.exceptions.PolicyError()
        zip_lib.unzip_zip(
            "tests/test_data/30-01-2019-RA-eLife-45644.zip", self.temp_dir
        )
        pdf_path = "tests/tmp/30-01-2019-RA-eLife-45644/Appendix 1figure 10.pdf"
        with self.assertRaises(wand.exceptions.PolicyError):
            self.assertIsNone(parse.pdf_page_count(pdf_path))
        expected = (
            "ERROR elifecleaner:parse:pdf_page_count: "
            "PolicyError in pdf_page_count(), "
            "imagemagick policy.xml may not allow reading PDF files\n"
        )
        log_file_lines = read_log_file_lines(self.log_file)
        self.assertEqual(log_file_lines[0], expected)


class TestParseArticleXML(unittest.TestCase):
    def setUp(self):
        self.temp_dir = "tests/tmp"

    def tearDown(self):
        delete_files_in_folder(self.temp_dir, filter_out=[".keepme"])

    def test_parse_article_xml(self):
        xml_file_path = os.path.join(self.temp_dir, "test.xml")
        with open(xml_file_path, "w") as open_file:
            open_file.write("<article/>")
        root = parse.parse_article_xml(xml_file_path)
        self.assertIsNotNone(root)

    def test_parse_article_xml_entities(self):
        xml_file_path = os.path.join(self.temp_dir, "test.xml")
        with open(xml_file_path, "w") as open_file:
            open_file.write("<article>&mdash;&lt;&gt;&amp;&quot;&beta;</article>")
        expected = b'<article>&#8212;&lt;&gt;&amp;"&#946;</article>'
        root = parse.parse_article_xml(xml_file_path)
        self.assertIsNotNone(root)
        self.assertEqual(ElementTree.tostring(root), expected)

    def test_parse_article_xml_control_character_entities(self):
        xml_file_path = os.path.join(self.temp_dir, "test.xml")
        with open(xml_file_path, "w") as open_file:
            open_file.write(
                "<article><title>To &#x001D;nd odd entities.</title></article>"
            )
        expected = b"<article><title>To %snd odd entities.</title></article>" % bytes(
            CONTROL_CHARACTER_ENTITY_REPLACEMENT, encoding="utf-8"
        )
        root = parse.parse_article_xml(xml_file_path)
        self.assertIsNotNone(root)
        self.assertEqual(ElementTree.tostring(root), expected)

    def test_parse_article_xml_control_characters(self):
        xml_file_path = os.path.join(self.temp_dir, "test.xml")
        with open(xml_file_path, "w") as open_file:
            open_file.write(
                "<article><title>To %snd odd entities.</title></article>" % chr(29)
            )
        expected = b"<article><title>To %snd odd entities.</title></article>" % bytes(
            CONTROL_CHARACTER_ENTITY_REPLACEMENT, encoding="utf-8"
        )
        root = parse.parse_article_xml(xml_file_path)
        self.assertIsNotNone(root)
        self.assertEqual(ElementTree.tostring(root), expected)

    def test_parse_article_xml_ampersand(self):
        xml_file_path = os.path.join(self.temp_dir, "test.xml")
        xml_string_pattern = "<article>And %s And &amp; &#8220;And&#8221;</article>"
        with open(xml_file_path, "w") as open_file:
            open_file.write(xml_string_pattern % "&")
        expected = bytes(xml_string_pattern % "&amp;", encoding="utf-8")
        root = parse.parse_article_xml(xml_file_path)
        self.assertIsNotNone(root)
        self.assertEqual(ElementTree.tostring(root), expected)

    def test_parse_article_xml_failure(self):
        xml_file_path = os.path.join(self.temp_dir, "test.xml")
        with open(xml_file_path, "w") as open_file:
            open_file.write("malformed xml")
        with self.assertRaises(ElementTree.ParseError):
            parse.parse_article_xml(xml_file_path)


class TestParseXML(unittest.TestCase):
    def setUp(self):
        xml_string = """<article>
<front>
  <journal-meta>
    <journal-id journal-id-type="foo">bar</journal-id>
    <journal-title-group>
      <journal-title>eLife</journal-title>
    </journal-title-group>
    <publisher>
        <publisher-name>eLife Sciences Publications, Ltd</publisher-name>
    </publisher>
  </journal-meta>
</front>
</article>"""
        self.root = ElementTree.fromstring(xml_string)

    def test_xml_journal_id_values(self):
        id_values = parse.xml_journal_id_values(self.root)
        self.assertEqual(id_values, {"foo": "bar"})

    def test_xml_journal_id_values_none(self):
        id_values = parse.xml_journal_id_values(ElementTree.fromstring("<article/>"))
        self.assertEqual(id_values, {})

    def test_xml_journal_title(self):
        title = parse.xml_journal_title(self.root)
        self.assertEqual(title, "eLife")

    def test_xml_journal_title_none(self):
        title = parse.xml_journal_title(ElementTree.fromstring("<article/>"))
        self.assertEqual(title, None)

    def test_xml_publisher_name(self):
        name = parse.xml_publisher_name(self.root)
        self.assertEqual(name, "eLife Sciences Publications, Ltd")

    def test_xml_publisher_name_none(self):
        name = parse.xml_publisher_name(ElementTree.fromstring("<article/>"))
        self.assertEqual(name, None)


class TestRepairArticleXml(unittest.TestCase):
    def test_malformed_xml(self):
        xml_string = "malformed xml"
        expected = "malformed xml"
        self.assertEqual(parse.repair_article_xml(xml_string), expected)

    def test_research_article(self):
        xml_string = '<article article-type="research-article"></article>'
        expected = (
            '<article article-type="research-article" '
            'xmlns:xlink="http://www.w3.org/1999/xlink"></article>'
        )
        self.assertEqual(parse.repair_article_xml(xml_string), expected)

    def test_article_commentary(self):
        xml_string = '<article article-type="article-commentary"></article>'
        expected = (
            '<article article-type="article-commentary" '
            'xmlns:xlink="http://www.w3.org/1999/xlink"></article>'
        )
        self.assertEqual(parse.repair_article_xml(xml_string), expected)

    def test_article_tag(self):
        xml_string = "<article></article>"
        expected = '<article xmlns:xlink="http://www.w3.org/1999/xlink"></article>'
        self.assertEqual(parse.repair_article_xml(xml_string), expected)

    def test_xlink_namespace_already_exists(self):
        xml_string = '<article xmlns:xlink="http://www.w3.org/1999/xlink"></article>'
        expected = '<article xmlns:xlink="http://www.w3.org/1999/xlink"></article>'
        self.assertEqual(parse.repair_article_xml(xml_string), expected)

    def test_artice_id_tag(self):
        # do not add to <article-id> tags
        xml_string = (
            '<article article-type="research-article"></article>'
            '<article-id pub-id-type="publisher-id">45644</article-id>'
        )
        expected = (
            '<article article-type="research-article" xmlns:xlink="http://www.w3.org/1999/xlink">'
            '</article><article-id pub-id-type="publisher-id">45644</article-id>'
        )
        self.assertEqual(parse.repair_article_xml(xml_string), expected)


class TestParseFileList(unittest.TestCase):
    def setUp(self):
        self.temp_dir = "tests/tmp"

    def tearDown(self):
        delete_files_in_folder(self.temp_dir, filter_out=[".keepme"])

    def test_file_list(self):
        zip_file = "tests/test_data/30-01-2019-RA-eLife-45644.zip"
        asset_file_name_map = zip_lib.unzip_zip(zip_file, self.temp_dir)
        xml_asset = parse.article_xml_asset(asset_file_name_map)
        root = parse.parse_article_xml(xml_asset[1])
        expected = read_fixture("file_list_45644.py")

        files = parse.file_list(root)
        self.assertEqual(files, expected)


class TestCheckMissingFiles(unittest.TestCase):
    def setUp(self):
        self.temp_dir = "tests/tmp"
        self.log_file = os.path.join(self.temp_dir, "test.log")
        self.log_handler = configure_logging(self.log_file)

    def tearDown(self):
        LOGGER.removeHandler(self.log_handler)
        delete_files_in_folder(self.temp_dir, filter_out=[".keepme"])

    @patch.object(parse, "find_missing_files")
    def test_check_missing_files(self, mock_missing_files):
        identifier = "test.zip"
        missing_files = ["file.jpg"]
        mock_missing_files.return_value = missing_files
        expected = [
            (
                "WARNING elifecleaner:parse:check_missing_files: %s "
                "does not contain a file in the manifest: %s\n"
            )
            % (identifier, missing_files[0])
        ]
        parse.check_missing_files([], {}, identifier)
        self.assertEqual(read_log_file_lines(self.log_file), expected)


class TestFindMissingFiles(unittest.TestCase):
    def test_find_missing_files_complete(self):
        files = []
        asset_file_name_map = {}
        expected = []
        self.assertEqual(parse.find_missing_files(files, asset_file_name_map), expected)

    def test_find_missing_files_incomplete(self):
        files = [
            OrderedDict(
                [
                    ("file_type", "figure"),
                    ("id", "2063134"),
                    ("upload_file_nm", "eLife64719_figure1_classB.png"),
                    (
                        "custom_meta",
                        [
                            OrderedDict(
                                [
                                    ("meta_name", "Figure number"),
                                    ("meta_value", "Figure 1"),
                                ]
                            )
                        ],
                    ),
                ]
            )
        ]
        asset_file_name_map = {}
        expected = ["eLife64719_figure1_classB.png"]
        self.assertEqual(parse.find_missing_files(files, asset_file_name_map), expected)


class TestCheckMultiPageFigurePdf(unittest.TestCase):
    def setUp(self):
        self.temp_dir = "tests/tmp"
        self.log_file = os.path.join(self.temp_dir, "test.log")
        self.log_handler = configure_logging(self.log_file)

    def tearDown(self):
        LOGGER.removeHandler(self.log_handler)
        delete_files_in_folder(self.temp_dir, filter_out=[".keepme"])

    @patch.object(pdf_utils, "pdf_image_pages")
    @patch.object(pdf_utils, "pdfimages_exists")
    def test_check_multi_page_figure_pdf(
        self, mock_pdfimages_exists, mock_pdf_image_pages
    ):
        figures = [{"file_name": "figure.pdf", "pages": 2}]
        zip_file = "30-01-2019-RA-eLife-45644.zip"
        mock_pdfimages_exists.return_value = False
        mock_pdf_image_pages.return_value = {1, 2}
        expected = None
        self.assertEqual(parse.check_multi_page_figure_pdf(figures, zip_file), expected)
        log_file_lines = read_log_file_lines(self.log_file)
        self.assertTrue(
            "multiple page PDF figure file: figure.pdf" in log_file_lines[0]
        )

    @patch.object(pdf_utils, "pdf_image_pages")
    @patch.object(pdf_utils, "pdfimages_exists")
    def test_check_multi_page_figure_pdf_exception(
        self, mock_pdfimages_exists, mock_pdf_image_pages
    ):
        figures = [{"file_name": "figure.pdf", "pages": 2}]
        zip_file = "30-01-2019-RA-eLife-45644.zip"
        mock_pdfimages_exists.return_value = True
        mock_pdf_image_pages.side_effect = Exception("An exception")
        expected = None
        self.assertEqual(parse.check_multi_page_figure_pdf(figures, zip_file), expected)
        log_file_lines = read_log_file_lines(self.log_file)
        self.assertTrue("Exception:" in log_file_lines[-2])
        self.assertTrue(
            "multiple page PDF figure file: figure.pdf" in log_file_lines[-1]
        )


class TestCheckExtraFiles(unittest.TestCase):
    def setUp(self):
        self.temp_dir = "tests/tmp"
        self.log_file = os.path.join(self.temp_dir, "test.log")
        self.log_handler = configure_logging(self.log_file)

    def tearDown(self):
        LOGGER.removeHandler(self.log_handler)
        delete_files_in_folder(self.temp_dir, filter_out=[".keepme"])

    @patch.object(parse, "find_extra_files")
    def test_check_extra_files(self, mock_extra_files):
        identifier = "test.zip"
        extra_files = ["file.jpg"]
        mock_extra_files.return_value = extra_files
        expected = [
            (
                "WARNING elifecleaner:parse:check_extra_files: %s "
                "has file not listed in the manifest: %s\n"
            )
            % (identifier, extra_files[0])
        ]
        parse.check_extra_files([], {}, identifier)
        self.assertEqual(read_log_file_lines(self.log_file), expected)


class TestFindExtraFiles(unittest.TestCase):
    def test_find_extra_files_empty(self):
        files = []
        asset_file_name_map = {}
        expected = []
        self.assertEqual(parse.find_extra_files(files, asset_file_name_map), expected)

    def test_find_extra_files_complete(self):
        files = [
            OrderedDict(
                [
                    ("file_type", "figure"),
                    ("id", "2063134"),
                    ("upload_file_nm", "eLife64719_figure1_classB.png"),
                    ("custom_meta", []),
                ]
            )
        ]
        asset_file_name_map = {
            "08-11-2020-FA-eLife-64719/eLife64719_figure1_classB.png": (
                "tests/tmp/08-11-2020-FA-eLife-64719/eLife64719_figure1_classB.png"
            )
        }
        expected = []
        self.assertEqual(parse.find_extra_files(files, asset_file_name_map), expected)

    def test_find_extra_files_extra(self):
        files = []
        asset_file_name_map = {
            "08-11-2020-FA-eLife-64719/eLife64719_figure1_classB.png": (
                "tests/tmp/08-11-2020-FA-eLife-64719/eLife64719_figure1_classB.png"
            )
        }
        expected = ["eLife64719_figure1_classB.png"]
        self.assertEqual(parse.find_extra_files(files, asset_file_name_map), expected)


class TestCheckMissingFilesByName(unittest.TestCase):
    def setUp(self):
        self.temp_dir = "tests/tmp"
        self.log_file = os.path.join(self.temp_dir, "test.log")
        self.log_handler = configure_logging(self.log_file)

    def tearDown(self):
        LOGGER.removeHandler(self.log_handler)
        delete_files_in_folder(self.temp_dir, filter_out=[".keepme"])

    @patch.object(parse, "find_missing_files_by_name")
    def test_check_missing_files_by_name(self, mock_missing_files):
        identifier = "test.zip"
        missing_files = ["Figure 2"]
        mock_missing_files.return_value = missing_files
        expected = [
            (
                "WARNING elifecleaner:parse:check_missing_files_by_name: %s "
                "has file missing from expected numeric sequence: %s\n"
            )
            % (identifier, missing_files[0])
        ]
        parse.check_missing_files_by_name([], identifier)
        self.assertEqual(read_log_file_lines(self.log_file), expected)


class TestFindMissingFilesByName(unittest.TestCase):
    def setUp(self):
        self.files = [
            OrderedDict(
                [
                    ("file_type", "figure"),
                    ("id", "2063134"),
                    ("upload_file_nm", "eLife64719_figure1_classB.png"),
                    (
                        "custom_meta",
                        [
                            OrderedDict(
                                [
                                    ("meta_name", "Figure number"),
                                    ("meta_value", "Figure 1"),
                                ]
                            )
                        ],
                    ),
                ]
            ),
            OrderedDict(
                [
                    ("file_type", "figure"),
                    ("id", "2063135"),
                    ("upload_file_nm", "eLife64719_figure2_classB.png"),
                    (
                        "custom_meta",
                        [
                            OrderedDict(
                                [
                                    ("meta_name", "Figure number"),
                                    ("meta_value", "Figure 2"),
                                ]
                            )
                        ],
                    ),
                ]
            ),
        ]

    def test_find_missing_files_by_name_empty(self):
        files = []
        expected = []
        self.assertEqual(parse.find_missing_files_by_name(files), expected)

    def test_find_missing_files_by_name_complete(self):
        expected = []
        self.assertEqual(parse.find_missing_files_by_name(self.files), expected)

    def test_find_missing_files_by_name_missing(self):
        self.files[1]["custom_meta"][0]["meta_value"] = "Figure 3"
        expected = ["Figure 2"]
        self.assertEqual(parse.find_missing_files_by_name(self.files), expected)

    def test_find_missing_files_by_name_extra_whitespace(self):
        "test pattern matching if there is whitespace"
        self.files[1]["custom_meta"][0]["meta_value"] = "Figure  \t     3"
        expected = ["Figure 2"]
        self.assertEqual(parse.find_missing_files_by_name(self.files), expected)

    def test_find_file_detail_values_empty(self):
        files = []
        file_types = []
        meta_names = []
        expected = []
        self.assertEqual(
            parse.find_file_detail_values(files, file_types, meta_names), expected
        )

    def test_find_file_detail_values(self):
        files = [
            {
                "file_type": "figure",
                "custom_meta": [
                    {
                        "meta_name": "Title",
                        "meta_value": "Figure 1",
                    }
                ],
            },
            {
                "file_type": "figure",
                "custom_meta": [
                    {
                        "meta_name": "Figure number",
                        "meta_value": "Figure 2",
                    },
                    {
                        "meta_name": "Title",
                        "meta_value": "Second value will be ignored",
                    },
                ],
            },
            {
                "file_type": "additional_figure_data",
                "custom_meta": [
                    {
                        "meta_name": "Figure number",
                        "meta_value": "Figure 2-figure supplement 1",
                    }
                ],
            },
            {
                "file_type": "not_a_figure",
                "custom_meta": [
                    {
                        "meta_name": "Title",
                        "meta_value": "Figure 3",
                    }
                ],
            },
        ]
        file_types = ["figure"]
        meta_names = ["Title", "Figure number"]
        expected = [("figure", "Figure 1"), ("figure", "Figure 2")]
        self.assertEqual(
            parse.find_file_detail_values(files, file_types, meta_names), expected
        )

    def test_find_missing_value_by_sequence(self):
        "test for matching double-digit numbers"
        match_pattern = r"Figure (\d+)"
        values = [
            "Figure 1",
            "Figure 2",
            "Figure 3",
            "Figure 4",
            "Figure 5",
            "Figure 6",
            "Figure 7",
            "Figure 8",
            "Figure 9",
            "Figure 10",
            "Figure 11",
            "Figure 13",
        ]

        expected = ["Figure 12"]
        self.assertEqual(
            parse.find_missing_value_by_sequence(values, match_pattern), expected
        )


class TestCheckArtFile(unittest.TestCase):
    def setUp(self):
        self.temp_dir = "tests/tmp"
        self.log_file = os.path.join(self.temp_dir, "test.log")
        self.log_handler = configure_logging(self.log_file)

    def tearDown(self):
        LOGGER.removeHandler(self.log_handler)
        delete_files_in_folder(self.temp_dir, filter_out=[".keepme"])

    def test_check_art_file(self):
        "test for an acceptable file type will produce no warning"
        identifier = "test.zip"
        files = [
            OrderedDict(
                [
                    ("file_type", "art_file"),
                    ("upload_file_nm", "test.Docx"),
                ]
            )
        ]
        expected = []
        parse.check_art_file(files, identifier)
        self.assertEqual(read_log_file_lines(self.log_file), expected)

    def test_check_art_file_bad_extension(self):
        "test an unacceptable file type will produce a warning"
        identifier = "test.zip"
        files = [
            OrderedDict(
                [
                    ("file_type", "art_file"),
                    ("upload_file_nm", "test.pdf"),
                ]
            )
        ]
        expected = [
            (
                (
                    "WARNING elifecleaner:parse:check_art_file: %s could not find a "
                    "word or latex article file in the package\n"
                )
            )
            % (identifier)
        ]
        parse.check_art_file(files, identifier)
        self.assertEqual(read_log_file_lines(self.log_file), expected)

    def test_check_art_file_missing(self):
        "test if there is no art_file it will produce a warning"
        identifier = "test.zip"
        files = [OrderedDict()]
        expected = [
            (
                (
                    "WARNING elifecleaner:parse:check_art_file: %s could not find a "
                    "word or latex article file in the package\n"
                )
            )
            % (identifier)
        ]
        parse.check_art_file(files, identifier)
        self.assertEqual(read_log_file_lines(self.log_file), expected)


class TestParsePreprintUrl(unittest.TestCase):
    def test_preprint_url(self):
        xml_string = """<article xmlns:xlink="http://www.w3.org/1999/xlink">
    <front>
        <article-meta>
            <fn-group content-type="article-history">
                <title>Preprint</title>
                <fn fn-type="other"/>
                <ext-link ext-link-type="url" xlink:href="https://doi.org/10.1101/2021.06.02.446694"/>
            </fn-group>
        </article-meta>
    </front>
</article>"""
        expected = "https://doi.org/10.1101/2021.06.02.446694"
        root = ElementTree.fromstring(xml_string)
        result = parse.preprint_url(root)
        self.assertEqual(result, expected)

    def test_preprint_url_no_article_meta(self):
        xml_string = """<article xmlns:xlink="http://www.w3.org/1999/xlink">
    <front></front>
</article>"""
        expected = None
        root = ElementTree.fromstring(xml_string)
        result = parse.preprint_url(root)
        self.assertEqual(result, expected)

    def test_preprint_url_no_ext_link(self):
        xml_string = """<article xmlns:xlink="http://www.w3.org/1999/xlink">
    <front>
        <article-meta>
            <fn-group content-type="article-history"></fn-group>
        </article-meta>
    </front>
</article>"""
        expected = None
        root = ElementTree.fromstring(xml_string)
        result = parse.preprint_url(root)
        self.assertEqual(result, expected)



import unittest
from xml.etree import ElementTree
from xml.etree.ElementTree import Element
from elifetools import xmlio
from elifecleaner import block


class TestMatchFigLabelContent(unittest.TestCase):
    "tests for block.match_fig_label_content()"

    def test_match_fig_label_content(self):
        "test matching paragraph content as a fig label"
        cases = [
            (None, False),
            ("", False),
            ("Test", False),
            ("Review image 1.", True),
            ("Review image 1", True),
            ("Review table 1.", False),
            ("Author response image 1.", True),
        ]
        for content, expected in cases:
            self.assertEqual(block.match_fig_label_content(content), expected)


class TestMatchTableLabelContent(unittest.TestCase):
    "tests for block.match_table_label_content()"

    def test_match_table_label_content(self):
        "test matching paragraph content as a table label"
        cases = [
            (None, False),
            ("", False),
            ("Test", False),
            ("Review table 1.", True),
            ("Review table 1", True),
            ("Review image 1.", False),
            ("Author response table 1.", True),
        ]
        for content, expected in cases:
            self.assertEqual(
                block.match_table_label_content(content), expected, 'on "%s"' % content
            )


class TestIsPLabel(unittest.TestCase):
    "tests for block.is_p_label()"

    def test_is_p_label(self):
        "example of a matched label in a p tag"
        xml_string = "<p><bold>Review image 1.</bold></p>"
        block_type = "fig"
        expected = True
        self.assertEqual(
            block.is_p_label(
                ElementTree.fromstring(xml_string), None, None, block_type, None
            ),
            expected,
        )

    def test_table_is_p_label(self):
        "example of a matched table label in a p tag"
        xml_string = "<p><bold>Review table 1.</bold></p>"
        block_type = "table"
        expected = True
        self.assertEqual(
            block.is_p_label(
                ElementTree.fromstring(xml_string), None, None, block_type, None
            ),
            expected,
        )

    def test_no_block_type(self):
        "test if block_type is None"
        xml_string = "<p><bold>Review table 1.</bold></p>"
        block_type = None
        expected = None
        self.assertEqual(
            block.is_p_label(
                ElementTree.fromstring(xml_string), None, None, block_type, None
            ),
            expected,
        )

    def test_false(self):
        "not a label in a p tag"
        xml_string = "<p>Foo.</p>"
        block_type = "fig"
        expected = False
        self.assertEqual(
            block.is_p_label(
                ElementTree.fromstring(xml_string), None, None, block_type, None
            ),
            expected,
        )


class TestIsPInlineGraphic(unittest.TestCase):
    "tests for block.is_p_inline_graphic()"

    def test_is_p_inline_graphic(self):
        "simple example of an inline-graphic p tag"
        xml_string = "<p><inline-graphic /></p>"
        expected = True
        self.assertEqual(
            block.is_p_inline_graphic(
                ElementTree.fromstring(xml_string), None, None, None
            ),
            expected,
        )

    def test_whitespace(self):
        "test if when whitespace surrounds the inline-graphic tag"
        xml_string = "<p>   <inline-graphic />      </p>"
        expected = True
        self.assertEqual(
            block.is_p_inline_graphic(
                ElementTree.fromstring(xml_string), None, None, None
            ),
            expected,
        )

    def test_false(self):
        "not an inline-graphic p tag"
        xml_string = "<p>Foo.</p>"
        expected = False
        self.assertEqual(
            block.is_p_inline_graphic(
                ElementTree.fromstring(xml_string), None, None, None
            ),
            expected,
        )

    def test_none(self):
        "an inline-graphic tag and other content returns None"
        xml_string = "<p>An <inline-graphic/></p>"
        expected = None
        self.assertEqual(
            block.is_p_inline_graphic(
                ElementTree.fromstring(xml_string), None, None, None
            ),
            expected,
        )


class TestSetGraphicTag(unittest.TestCase):
    "tests for block.set_graphic_tag()"

    def test_set_graphic_tag(self):
        "create graphic tag for a fig"
        # register XML namespaces
        xmlio.register_xmlns()
        xml_string = '<fig xmlns:xlink="http://www.w3.org/1999/xlink" />'
        tag = ElementTree.fromstring(xml_string)
        image_href = "image.png"
        new_file_name = "new_image.png"
        expected = (
            b'<fig xmlns:xlink="http://www.w3.org/1999/xlink">'
            b'<graphic mimetype="image" mime-subtype="png" xlink:href="new_image.png" />'
            b"</fig>"
        )
        block.set_graphic_tag(tag, image_href, new_file_name)
        self.assertEqual(ElementTree.tostring(tag), expected)


class TestSetCaptionTag(unittest.TestCase):
    "tests for block.set_caption_tag()"

    def test_set_caption_tag(self):
        "create caption tag for a fig"
        body_tag_xml_string = (
            b"<body>"
            b"<p>Title <italic>here</italic>. "
            b"Caption paragraph <italic>ici</italic>."
            b"</p>"
            b"</body>"
        )
        body_tag = ElementTree.fromstring(body_tag_xml_string)
        tag = Element("fig")
        caption_index = 0
        expected = (
            b"<fig>"
            b"<caption>"
            b"<title>Title <italic>here</italic>.</title>"
            b"<p>Caption paragraph <italic>ici</italic>.</p>"
            b"</caption>"
            b"</fig>"
        )
        block.set_caption_tag(tag, body_tag, caption_index)
        self.assertEqual(ElementTree.tostring(tag), expected)


class TestSubArticleTagParts(unittest.TestCase):
    "tests for block.sub_article_tag_parts()"

    def test_sub_article_tag_parts(self):
        "test parsing sub-article XML tag data"
        id_value = "sa0"
        xml_string = '<sub-article id="%s"><body/></sub-article>' % id_value
        sub_article_root = ElementTree.fromstring(xml_string)
        result = block.sub_article_tag_parts(sub_article_root)
        self.assertEqual(result[0], id_value)
        self.assertTrue(isinstance(result[1], Element))
        self.assertEqual(result[1].tag, "body")


class TestTagIndexGroups(unittest.TestCase):
    "tests for block.tag_index_groups()"

    def setUp(self):
        self.sub_article_id = "sa0"
        self.identifier = "test.zip"

    def test_tag_index_groups(self):
        "test body XML tag with fig content"
        xml_string = (
            '<body xmlns:xlink="http://www.w3.org/1999/xlink">'
            "<blockquote><p>Quotation.</p></blockquote>"
            "<p><bold>Author response image 1.</bold></p>"
            "<p>Caption title. Caption paragraph <italic>here</italic>.</p>"
            '<p><inline-graphic xlink:href="test.png"/></p>'
            "</body>"
        )
        block_type = "fig"
        expected = [{"label_index": 1, "caption_index": 2, "inline_graphic_index": 3}]
        body_tag = ElementTree.fromstring(xml_string)
        result = block.tag_index_groups(
            body_tag, self.sub_article_id, block_type, self.identifier
        )
        self.assertDictEqual(result[0], expected[0])

    def test_table_no_caption(self):
        "test body XML tag with a table and no caption"
        xml_string = (
            '<body xmlns:xlink="http://www.w3.org/1999/xlink">'
            "<p><bold>Author response table 1.</bold></p>"
            '<p><inline-graphic xlink:href="test.png"/></p>'
            "</body>"
        )
        block_type = "table"
        expected = {"label_index": 0, "caption_index": False, "inline_graphic_index": 1}
        body_tag = ElementTree.fromstring(xml_string)
        result = block.tag_index_groups(
            body_tag, self.sub_article_id, block_type, self.identifier
        )
        self.assertDictEqual(result[0], expected)

    def test_empty_body_tag(self):
        "test body XML tag with no tags inside"
        xml_string = "<body></body>"
        block_type = "fig"
        expected = []
        body_tag = ElementTree.fromstring(xml_string)
        result = block.tag_index_groups(
            body_tag, self.sub_article_id, block_type, self.identifier
        )
        self.assertEqual(result, expected)


class TestTitleParagraphContent(unittest.TestCase):
    "tests for block.title_paragraph_content()"

    def test_title_paragraph_content(self):
        "a simple example of string parts"
        string_list = [
            "<p>Caption title.",
            " Caption paragraph.",
            "</p>",
        ]
        expected = ("<p>Caption title.", " Caption paragraph.</p>")
        result = block.title_paragraph_content(string_list)
        self.assertEqual(result, expected)

    def test_multiple_sentences(self):
        "a simple example of string parts"
        string_list = [
            "<p>A title.",
            " A caption paragraph.",
            " Another paragraph.",
            "</p>",
        ]
        expected = ("<p>A title.", " A caption paragraph. Another paragraph.</p>")
        result = block.title_paragraph_content(string_list)
        self.assertEqual(result, expected)

    def test_italic(self):
        "test where an unmatched inline formatting tag is repaired"
        string_list = [
            "<p>The title<bold>.",
            "</bold> Another <bold>bold term</bold>.",
            " Another paragraph.",
            "</p>",
        ]
        expected = (
            "<p>The title<bold>.</bold> Another <bold>bold term</bold>.",
            " Another paragraph.</p>",
        )
        result = block.title_paragraph_content(string_list)
        self.assertEqual(result, expected)

    def test_empty_list(self):
        "test an empty list"
        string_list = [""]
        expected = ("", None)
        result = block.title_paragraph_content(string_list)
        self.assertEqual(result, expected)


class TestCaptionTitleParagraph(unittest.TestCase):
    "tests for block.caption_title_paragraph()"

    def setUp(self):
        # register XML namespaces
        xmlio.register_xmlns()

    def test_simple_title(self):
        "test a simple example"
        xml_string = "<p>Title. Caption.</p>"
        tag = ElementTree.fromstring(xml_string)
        caption_title_p_tag, caption_paragraph_p_tag = block.caption_title_paragraph(
            tag
        )
        self.assertEqual(ElementTree.tostring(caption_title_p_tag), b"<p>Title.</p>")
        self.assertEqual(
            ElementTree.tostring(caption_paragraph_p_tag), b"<p>Caption.</p>"
        )

    def test_organism_title(self):
        "test for italic tag included"
        xml_string = "<p>In <italic>B. subtilis</italic>, the title. Caption.</p>"
        tag = ElementTree.fromstring(xml_string)
        caption_title_p_tag, caption_paragraph_p_tag = block.caption_title_paragraph(
            tag
        )
        self.assertEqual(
            ElementTree.tostring(caption_title_p_tag),
            b"<p>In <italic>B. subtilis</italic>, the title.</p>",
        )
        self.assertEqual(
            ElementTree.tostring(caption_paragraph_p_tag), b"<p>Caption.</p>"
        )

    def test_two_bold_tags(self):
        "edge case with more than one bold tag and second one is around the title full stop"
        xml_string = (
            "<p>The title<bold>.</bold> Another <bold>bold term</bold>."
            " Another paragraph.</p>"
        )
        tag = ElementTree.fromstring(xml_string)
        caption_title_p_tag, caption_paragraph_p_tag = block.caption_title_paragraph(
            tag
        )
        self.assertEqual(
            ElementTree.tostring(caption_title_p_tag),
            b"<p>The title<bold>.</bold> Another <bold>bold term</bold>.</p>",
        )
        self.assertEqual(
            ElementTree.tostring(caption_paragraph_p_tag), b"<p>Another paragraph.</p>"
        )

    def test_ext_link_tag(self):
        "edge case with an ext-link tag in the title and challenging full stop separations"
        xml_string = '<p xmlns:xlink="http://www.w3.org/1999/xlink">(Figure 2A from <ext-link ext-link-type="uri" xlink:href="https://example.org/one/two">(Anonymous et al., 2011)</ext-link>)<bold>.</bold> Comparison of Tests against Γ ( 4,5) (a = 0.05). The normality tests used were severely underpowered for n&lt;100 (<ext-link ext-link-type="uri" xlink:href="https://example.org/one/two">(Anonymous et al., 2011)</ext-link>).</p>'
        tag = ElementTree.fromstring(xml_string)
        caption_title_p_tag, caption_paragraph_p_tag = block.caption_title_paragraph(
            tag
        )
        self.assertEqual(
            ElementTree.tostring(caption_title_p_tag),
            b'<p xmlns:xlink="http://www.w3.org/1999/xlink">(Figure 2A from <ext-link ext-link-type="uri" xlink:href="https://example.org/one/two">(Anonymous et al., 2011)</ext-link>)<bold>.</bold> Comparison of Tests against &#915; ( 4,5) (a = 0.</p>',
        )
        self.assertEqual(
            ElementTree.tostring(caption_paragraph_p_tag),
            b'<p xmlns:xlink="http://www.w3.org/1999/xlink">05). The normality tests used were severely underpowered for n&lt;100 (<ext-link ext-link-type="uri" xlink:href="https://example.org/one/two">(Anonymous et al., 2011)</ext-link>).</p>',
        )

    def test_mml_tag(self):
        "edge case where a full stop in math formula should not be used as separate parts"
        xml_string = '<p xmlns:mml="http://www.w3.org/1998/Math/MathML">For one participant <inline-formula><mml:math alttext="" display="inline"><mml:mspace width="0.222em" /></mml:math></inline-formula> is a formula.</p>'
        tag = ElementTree.fromstring(xml_string)
        caption_title_p_tag, caption_paragraph_p_tag = block.caption_title_paragraph(
            tag
        )
        self.assertEqual(
            ElementTree.tostring(caption_title_p_tag),
            b'<p xmlns:mml="http://www.w3.org/1998/Math/MathML">For one participant <inline-formula><mml:math alttext="" display="inline"><mml:mspace width="0.222em" /></mml:math></inline-formula> is a formula.</p>',
        )
        self.assertEqual(ElementTree.tostring(caption_paragraph_p_tag), b"<p />")


class TestSplitTtitleParts(unittest.TestCase):
    "tests for block.split_title_parts()"

    def test_split_title_parts(self):
        "test spliting a simple example"
        xml_string = "<p>A title. A caption paragraph.</p>"
        expected = ["<p>A title.", " A caption paragraph.", "</p>"]
        result = block.split_title_parts(xml_string)
        self.assertEqual(result, expected)

    def test_multiple_sentences(self):
        "test spliting a simple example"
        xml_string = "<p>A title. A caption paragraph. Another paragraph.</p>"
        expected = [
            "<p>A title.",
            " A caption paragraph.",
            " Another paragraph.",
            "</p>",
        ]
        result = block.split_title_parts(xml_string)
        self.assertEqual(result, expected)

    def test_namespace(self):
        "test spliting a simple example"
        xml_string = (
            '<p xmlns:xlink="http://www.w3.org/1999/xlink">'
            'A title only (<ext-link xlink:href="http://example.org"/>).'
            "</p>"
        )
        expected = [
            '<p xmlns:xlink="http://www.w3.org/1999/xlink">A title only (<ext-link '
            'xlink:href="http://example.org"/>).',
            "</p>",
        ]
        result = block.split_title_parts(xml_string)
        self.assertEqual(result, expected)

    def test_unmatched_tags(self):
        "example where there are unmatched tags in the result"
        xml_string = xml_string = (
            "<p>The title<bold>.</bold> Another <bold>bold term</bold>."
            " Another paragraph.</p>"
        )
        expected = [
            "<p>The title<bold>.",
            "</bold> Another <bold>bold term</bold>.",
            " Another paragraph.",
            "</p>",
        ]
        result = block.split_title_parts(xml_string)
        self.assertEqual(result, expected)

    def test_blank_string(self):
        "test splitting a blank string"
        xml_string = ""
        expected = []
        result = block.split_title_parts(xml_string)
        self.assertEqual(result, expected)

    def test_no_full_stop(self):
        "test splitting string wiht no full stop"
        xml_string = "<p>Test</p>"
        expected = ["<p>Test</p>"]
        result = block.split_title_parts(xml_string)
        self.assertEqual(result, expected)


class TestStripTagText(unittest.TestCase):
    "tests for block.strip_tag_text()"

    def test_strip_tag_text(self):
        "test an example with whitespace to the left of the tag"
        xml_string = b"<p> <inline-graphic /> </p>"
        expected = b"<p><inline-graphic /> </p>"
        p_tag = ElementTree.fromstring(xml_string)
        block.strip_tag_text(p_tag)
        self.assertEqual(ElementTree.tostring(p_tag), expected)

    def test_non_element(self):
        "test if the input is not a tag"
        tag = "foo"
        self.assertEqual(block.strip_tag_text(tag), None)


class TestStripTagTail(unittest.TestCase):
    "tests for block.strip_tag_tail()"

    def test_strip_tag_tail(self):
        "test an example with whitespace to the left and right of the tag"
        xml_string = b"<p> <inline-graphic /> </p>"
        expected = b"<inline-graphic />"
        p_tag = ElementTree.fromstring(xml_string)
        inline_graphic_tag = p_tag.find("inline-graphic")
        block.strip_tag_tail(inline_graphic_tag)
        self.assertEqual(ElementTree.tostring(inline_graphic_tag), expected)

    def test_non_element(self):
        "test if the input is not a tag"
        tag = "foo"
        self.assertEqual(block.strip_tag_tail(tag), None)


class TestInlineGraphicTagFromTag(unittest.TestCase):
    "tests for block.inline_graphic_tag_from_tag()"

    def test_tag_found(self):
        "test getting a inline-graphic tag from a p tag"
        xml_string = b"<p>   <inline-graphic />   </p>"
        tag = ElementTree.fromstring(xml_string)
        expected = b"<inline-graphic />"
        result = block.inline_graphic_tag_from_tag(tag)
        self.assertEqual(ElementTree.tostring(result), expected)


class TestSetLabelTag(unittest.TestCase):
    "tests for block.set_label_tag()"

    def test_set_label_tag(self):
        "create label tag for a fig"
        body_tag_xml_string = b"<body><p><bold>Label</bold></p></body>"
        body_tag = ElementTree.fromstring(body_tag_xml_string)
        tag = Element("fig")
        label_index = 0
        expected = b"<fig><label>Label</label></fig>"
        block.set_label_tag(tag, body_tag, label_index)
        self.assertEqual(ElementTree.tostring(tag), expected)



import os
import unittest
from xml.etree import ElementTree
from elifetools import xmlio
from elifecleaner import configure_logging, fig, LOGGER
from tests.helpers import (
    delete_files_in_folder,
    read_log_file_lines,
    sub_article_xml_fixture,
)


class TestFigFileNameIdentifer(unittest.TestCase):
    "tests for fig.fig_file_name_identifier()"

    def test_fig_file_name_identifier(self):
        "generate a fig graphic id value"
        sub_article_id = "sa2"
        fig_index = "1"
        expected = "sa2-fig1"
        self.assertEqual(
            fig.fig_file_name_identifier(sub_article_id, fig_index), expected
        )


class TestFigFileName(unittest.TestCase):
    "tests for fig.fig_file_name()"

    def test_fig_file_name(self):
        "generate a fig graphic file name from an inline-graphic file name"
        inf_file_name = "elife-70493-inf1.png"
        sub_article_id = "sa2"
        fig_index = "1"
        expected = "elife-70493-sa2-fig1.png"
        self.assertEqual(
            fig.fig_file_name(inf_file_name, sub_article_id, fig_index),
            expected,
        )


class TestFigTagIndexGroups(unittest.TestCase):
    "tests for fig.fig_tag_index_groups()"

    def test_fig_tag_index_groups(self):
        "test finding a group of p tags to be converted to a fig"
        xml_string = (
            b'<body xmlns:xlink="http://www.w3.org/1999/xlink">'
            b"<p>First paragraph.</p>"
            b"<p><bold>Review image 1.</bold></p>"
            b"<p>This is the caption for this image that describes what it contains.</p>"
            b'<p> <inline-graphic xlink:href="elife-70493-inf1.png" /> </p>'
            b"<p>Another paragraph with an inline graphic "
            b'<inline-graphic xlink:href="elife-70493-inf2.jpg" /></p>'
            b"</body>"
        )
        expected = [{"label_index": 1, "caption_index": 2, "inline_graphic_index": 3}]
        result = fig.fig_tag_index_groups(
            ElementTree.fromstring(xml_string), None, None
        )
        self.assertEqual(len(result), 1)
        self.assertDictEqual(result[0], expected[0])

    def test_multiple_figs(self):
        "test multiple groups of p tags to be converted to fig tags"
        xml_string = (
            b'<body xmlns:xlink="http://www.w3.org/1999/xlink">'
            b"<p>First paragraph.</p>"
            b"<p><bold>Review image 1.</bold></p>"
            b"<p>This is the caption for this image that describes what it contains.</p>"
            b'<p> <inline-graphic xlink:href="elife-70493-inf1.png" /> </p>'
            b"<p><bold>Review image 2</bold></p>"
            b'<p><inline-graphic xlink:href="elife-70493-inf2.jpg" /></p>'
            b"</body>"
        )
        expected = [
            {"label_index": 1, "caption_index": 2, "inline_graphic_index": 3},
            {"label_index": 4, "caption_index": False, "inline_graphic_index": 5},
        ]
        result = fig.fig_tag_index_groups(
            ElementTree.fromstring(xml_string), None, None
        )
        self.assertEqual(len(result), 2)
        # first fig
        self.assertDictEqual(result[0], expected[0])
        # second fig
        self.assertDictEqual(result[1], expected[1])

    def test_empty(self):
        "test if no p tag groups are found"
        xml_string = (
            b'<body xmlns:xlink="http://www.w3.org/1999/xlink">'
            b"<p>First paragraph.</p>"
            b"</body>"
        )
        result = fig.fig_tag_index_groups(
            ElementTree.fromstring(xml_string), None, None
        )
        self.assertEqual(len(result), 0)

    def test_none(self):
        "test None as input to finding p tag groups"
        result = fig.fig_tag_index_groups(None, None, None)
        self.assertEqual(result, [])


class TestTransformFigGroups(unittest.TestCase):
    "tests for fig.transform_fig_groups()"

    def test_transform_fig_groups(self):
        "give body tag and indexes of p tags to transform to a fig tag"
        # register XML namespaces
        xmlio.register_xmlns()
        xml_string = (
            b'<body xmlns:xlink="http://www.w3.org/1999/xlink">'
            b"<p>First paragraph.</p>"
            b"<p><bold>Review image 1.</bold></p>"
            b"<p>This is the caption for this image that describes what it contains.</p>"
            b'<p> <inline-graphic xlink:href="elife-70493-inf1.png" /> </p>'
            b"<p><bold>Review image 2</bold></p>"
            b'<p><inline-graphic xlink:href="elife-70493-inf2.jpg" /></p>'
            b"</body>"
        )
        body_tag = ElementTree.fromstring(xml_string)
        fig_index_groups = [
            {"label_index": 1, "caption_index": 2, "inline_graphic_index": 3},
            {"label_index": 4, "caption_index": False, "inline_graphic_index": 5},
        ]
        sub_article_id = "sa1"
        expected = (
            b'<body xmlns:xlink="http://www.w3.org/1999/xlink">'
            b"<p>First paragraph.</p>"
            b'<fig id="sa1fig1">'
            b"<label>Review image 1.</label>"
            b"<caption>"
            b"<title>"
            b"This is the caption for this image that describes what it contains."
            b"</title>"
            b"</caption>"
            b'<graphic mimetype="image" mime-subtype="png" xlink:href="elife-70493-sa1-fig1.png" />'
            b"</fig>"
            b'<fig id="sa1fig2">'
            b"<label>Review image 2</label>"
            b'<graphic mimetype="image" mime-subtype="jpg" xlink:href="elife-70493-sa1-fig2.jpg" />'
            b"</fig>"
            b"</body>"
        )
        fig.transform_fig_groups(body_tag, fig_index_groups, sub_article_id)
        self.assertEqual(ElementTree.tostring(body_tag), expected)


class TestRemoveTagAttributes(unittest.TestCase):
    "tests for fig.remove_tag_attributes()"

    def test_remove_tag_attributes(self):
        "test cleaning attributes from a p tag"
        # register XML namespaces
        xmlio.register_xmlns()
        xml_string = (
            b'<p xmlns:xlink="http://www.w3.org/1999/xlink" id="p" xlink:href="foo" />'
        )
        tag = ElementTree.fromstring(xml_string)
        expected = b"<p />"
        fig.remove_tag_attributes(tag)
        self.assertEqual(ElementTree.tostring(tag), expected)

    def test_non_element(self):
        "test if the input is not a tag"
        tag = "foo"
        self.assertEqual(fig.remove_tag_attributes(tag), None)


class TestInlineGraphicHrefs(unittest.TestCase):
    "tests for fig.inline_graphic_hrefs()"

    def test_inline_graphic_hrefs(self):
        "get a list of xlink:href values from inline-graphic tags to be converted to fig"
        xml_string = (
            b'<sub-article id="sa1" xmlns:xlink="http://www.w3.org/1999/xlink">'
            b"<body>"
            b"<p><bold>Review image 1.</bold></p>"
            b'<p><inline-graphic xlink:href="elife-70493-inf1.png"/></p>'
            b"<p>Next paragraph is not an inline-graphic href.</p>"
            b'<p><inline-graphic xlink:href="elife-70493-inf2.png"/></p>'
            b"</body>"
            b"</sub-article>"
        )
        identifier = "test.zip"
        tag = ElementTree.fromstring(xml_string)
        expected = ["elife-70493-inf1.png"]
        result = fig.inline_graphic_hrefs(tag, identifier)
        self.assertEqual(result, expected)


class TestGraphicHrefs(unittest.TestCase):
    "tests for fig.graphic_hrefs()"

    def test_graphic_hrefs(self):
        "get a list of xlink:href values from graphic tags"
        xml_string = (
            b'<sub-article id="sa1" xmlns:xlink="http://www.w3.org/1999/xlink"><body>'
            b"<p>"
            b'<fig id="sa1fig1">'
            b"<label>Review image 1.</label>"
            b"<caption>"
            b"<title>A title.</title>"
            b"<p>A caption.</p>"
            b"</caption>"
            b'<graphic mimetype="image" mime-subtype="png" xlink:href="elife-70493-sa1-fig1.png" />'
            b"</fig>"
            b"</p>"
            b"</body></sub-article>"
        )
        identifier = "test.zip"
        tag = ElementTree.fromstring(xml_string)
        expected = ["elife-70493-sa1-fig1.png"]
        result = fig.graphic_hrefs(tag, identifier)
        self.assertEqual(result, expected)

    def test_graphic_hrefs_no_match(self):
        "empty list of xlink:href values when there is no graphic tag"
        xml_string = (
            b'<sub-article id="sa1" xmlns:xlink="http://www.w3.org/1999/xlink">'
            b"<body><p/></body>"
            b"</sub-article>"
        )
        identifier = "test.zip"
        tag = ElementTree.fromstring(xml_string)
        expected = []
        result = fig.graphic_hrefs(tag, identifier)
        self.assertEqual(result, expected)


class TestTransformFigGroup(unittest.TestCase):
    "tests for fig.transform_fig_group()"

    def test_transform_fig_group(self):
        "give body tag and indexes of p tags to transform to a fig tag"
        # register XML namespaces
        xmlio.register_xmlns()
        xml_string = (
            b'<body xmlns:xlink="http://www.w3.org/1999/xlink">'
            b"<p>First paragraph.</p>"
            b"<p><bold>Review image 1.</bold></p>"
            b"<p>This is the caption for this image that describes what it contains.</p>"
            b'<p> <inline-graphic xlink:href="elife-70493-inf1.png" /> </p>'
            b"</body>"
        )
        body_tag = ElementTree.fromstring(xml_string)
        fig_index = 1
        fig_group = {"label_index": 1, "caption_index": 2, "inline_graphic_index": 3}
        sub_article_id = "sa1"
        expected = (
            b'<body xmlns:xlink="http://www.w3.org/1999/xlink">'
            b"<p>First paragraph.</p>"
            b'<fig id="sa1fig1">'
            b"<label>Review image 1.</label>"
            b"<caption>"
            b"<title>"
            b"This is the caption for this image that describes what it contains."
            b"</title>"
            b"</caption>"
            b'<graphic mimetype="image" mime-subtype="png" xlink:href="elife-70493-sa1-fig1.png" />'
            b"</fig>"
            b"</body>"
        )

        fig.transform_fig_group(body_tag, fig_index, fig_group, sub_article_id)
        self.assertEqual(ElementTree.tostring(body_tag), expected)


def sa1_sub_article_xml_fixture():
    "generate XML test fixture for a referee-report"
    article_type = b"referee-report"
    sub_article_id = b"sa1"
    front_stub_xml_string = (
        b"<front-stub>"
        b'<article-id pub-id-type="doi">10.7554/eLife.79713.1.sa1</article-id>'
        b"<title-group>"
        b"<article-title>Reviewer #1 (Public Review):</article-title>"
        b"</title-group>"
        b"</front-stub>"
    )
    body_xml_string = (
        b"<body>"
        b"<p>First paragraph.</p>"
        b"<p><bold>Review image 1.</bold></p>"
        b"<p>This is the caption for this image that describes what it contains.</p>"
        b'<p> <inline-graphic xlink:href="elife-70493-inf1.png" /> </p>'
        b"<p>Another paragraph with an inline graphic "
        b'<inline-graphic xlink:href="elife-70493-inf2.jpg" /></p>'
        b"</body>"
    )
    return sub_article_xml_fixture(
        article_type, sub_article_id, front_stub_xml_string, body_xml_string
    )


def sa4_sub_article_xml_fixture(body_xml_string):
    "generate XML test fixture for an author-comment"
    article_type = b"author-comment"
    sub_article_id = b"sa4"
    front_stub_xml_string = (
        b"<front-stub>"
        b'<article-id pub-id-type="doi">10.7554/eLife.79713.1.sa4</article-id>'
        b"<title-group><article-title>Author response</article-title></title-group>"
        b"</front-stub>"
    )
    return sub_article_xml_fixture(
        article_type, sub_article_id, front_stub_xml_string, body_xml_string
    )


class TestTransformFig(unittest.TestCase):
    "tests for fig.transform_fig()"

    def setUp(self):
        self.identifier = "test.zip"
        self.temp_dir = "tests/tmp"
        self.log_file = os.path.join(self.temp_dir, "test.log")
        self.log_handler = configure_logging(self.log_file)

    def tearDown(self):
        LOGGER.removeHandler(self.log_handler)
        delete_files_in_folder(self.temp_dir, filter_out=[".keepme"])

    def test_referee_report(self):
        "convert tags to a fig in a referee-report fixture"
        # register XML namespaces
        xmlio.register_xmlns()
        xml_string = sa1_sub_article_xml_fixture()
        sub_article_root = ElementTree.fromstring(xml_string)
        root = fig.transform_fig(sub_article_root, self.identifier)

        rough_xml_string = ElementTree.tostring(root, "utf-8")
        self.assertTrue("<p>First paragraph.</p>" in rough_xml_string.decode("utf8"))
        self.assertTrue(
            '<inline-graphic xlink:href="elife-70493-inf2.jpg" />'
            in rough_xml_string.decode("utf8")
        )
        print(rough_xml_string.decode("utf8"))
        self.assertTrue(
            (
                "<body>"
                "<p>First paragraph.</p>"
                '<fig id="sa1fig1">'
                "<label>Review image 1.</label>"
                "<caption>"
                "<title>"
                "This is the caption for this image that describes what it contains."
                "</title>"
                "</caption>"
                '<graphic mimetype="image" mime-subtype="png" '
                'xlink:href="elife-70493-sa1-fig1.png" />'
                "</fig>"
                "<p>Another paragraph with an inline graphic "
                '<inline-graphic xlink:href="elife-70493-inf2.jpg" />'
                "</p>"
                "</body>"
            )
            in rough_xml_string.decode("utf8")
        )
        info_prefix = "INFO elifecleaner:block"
        expected = [
            '%s:%s: %s potential label "Review image 1." in p tag 1 of id sa1\n'
            % (info_prefix, "is_p_label", self.identifier),
            "%s:%s: %s label p tag index 1 of id sa1\n"
            % (info_prefix, "tag_index_groups", self.identifier),
            "%s:%s: %s no inline-graphic tag found in p tag 2 of id sa1\n"
            % (info_prefix, "is_p_inline_graphic", self.identifier),
            "%s:%s: %s only inline-graphic tag found in p tag 3 of id sa1\n"
            % (info_prefix, "is_p_inline_graphic", self.identifier),
        ]
        self.assertEqual(read_log_file_lines(self.log_file), expected)

    def test_author_response(self):
        "convert tags to a fig in an author-comment fixture"
        # register XML namespaces
        xmlio.register_xmlns()
        body_xml_string = (
            b"<body>"
            b"<p>First paragraph.</p>"
            b"<p><bold>Author response image 1.</bold></p>"
            b"<p>This is the caption for this image that describes what it contains.</p>"
            b'<p> <inline-graphic xlink:href="elife-70493-inf3.png" /> </p>'
            b"<p>Another paragraph with an inline graphic "
            b'<inline-graphic xlink:href="elife-70493-inf4.jpg" /></p>'
            b"</body>"
        )
        xml_string = sa4_sub_article_xml_fixture(body_xml_string)

        sub_article_root = ElementTree.fromstring(xml_string)
        root = fig.transform_fig(sub_article_root, self.identifier)

        rough_xml_string = ElementTree.tostring(root, "utf-8")
        self.assertTrue("<p>First paragraph.</p>" in rough_xml_string.decode("utf8"))
        self.assertTrue(
            '<inline-graphic xlink:href="elife-70493-inf4.jpg" />'
            in rough_xml_string.decode("utf8")
        )
        self.assertTrue(
            (
                "<body>"
                "<p>First paragraph.</p>"
                '<fig id="sa4fig1">'
                "<label>Author response image 1.</label>"
                "<caption>"
                "<title>"
                "This is the caption for this image that describes what it contains."
                "</title>"
                "</caption>"
                '<graphic mimetype="image" mime-subtype="png" '
                'xlink:href="elife-70493-sa4-fig1.png" />'
                "</fig>"
                "<p>Another paragraph with an inline graphic "
                '<inline-graphic xlink:href="elife-70493-inf4.jpg" />'
                "</p>"
                "</body>"
            )
            in rough_xml_string.decode("utf8")
        )
        info_prefix = "INFO elifecleaner:block"
        expected = [
            '%s:%s: %s potential label "Author response image 1." in p tag 1 of id sa4\n'
            % (info_prefix, "is_p_label", self.identifier),
            "%s:%s: %s label p tag index 1 of id sa4\n"
            % (info_prefix, "tag_index_groups", self.identifier),
            "%s:%s: %s no inline-graphic tag found in p tag 2 of id sa4\n"
            % (info_prefix, "is_p_inline_graphic", self.identifier),
            "%s:%s: %s only inline-graphic tag found in p tag 3 of id sa4\n"
            % (info_prefix, "is_p_inline_graphic", self.identifier),
        ]
        self.assertEqual(read_log_file_lines(self.log_file), expected)

    def test_caption_italic(self):
        "convert tags to a fig where the caption has an inline intalic tag"
        # register XML namespaces
        xmlio.register_xmlns()

        body_xml_string = (
            b"<body>"
            b"<p>First paragraph.</p>"
            b"<p><bold>Author response image 1.</bold></p>"
            b'<p>Caption with an <italic id="foo">italic</italic> tag or '
            b"<bold>bold</bold> too.</p>"
            b'<p> <inline-graphic xlink:href="elife-70493-inf3.png" /> </p>'
            b"<p>Another paragraph with an inline graphic "
            b'<inline-graphic xlink:href="elife-70493-inf4.jpg" /></p>'
            b"</body>"
        )

        xml_string = sa4_sub_article_xml_fixture(body_xml_string)
        sub_article_root = ElementTree.fromstring(xml_string)
        root = fig.transform_fig(sub_article_root, self.identifier)

        rough_xml_string = ElementTree.tostring(root, "utf-8")
        self.assertTrue(
            (
                "<body>"
                "<p>First paragraph.</p>"
                '<fig id="sa4fig1">'
                "<label>Author response image 1.</label>"
                "<caption>"
                "<title>"
                'Caption with an <italic id="foo">italic</italic> tag or <bold>bold</bold> too.'
                "</title>"
                "</caption>"
                '<graphic mimetype="image" mime-subtype="png" '
                'xlink:href="elife-70493-sa4-fig1.png" />'
                "</fig>"
                "<p>Another paragraph with an inline graphic "
                '<inline-graphic xlink:href="elife-70493-inf4.jpg" />'
                "</p>"
                "</body>"
            )
            in rough_xml_string.decode("utf8")
        )

    def test_no_caption(self):
        "convert tags to a fig where there is no p tag for the caption content"
        # register XML namespaces
        xmlio.register_xmlns()
        body_xml_string = (
            b"<body>"
            b"<p>First paragraph.</p>"
            b"<p><bold>Author response image 1.</bold></p>"
            b'<p> <inline-graphic xlink:href="elife-70493-inf3.png" /> </p>'
            b"<p>Another paragraph with an inline graphic "
            b'<inline-graphic xlink:href="elife-70493-inf4.jpg" /></p>'
            b"</body>"
        )
        xml_string = sa4_sub_article_xml_fixture(body_xml_string)

        sub_article_root = ElementTree.fromstring(xml_string)
        root = fig.transform_fig(sub_article_root, self.identifier)

        rough_xml_string = ElementTree.tostring(root, "utf-8")
        self.assertTrue(
            (
                "<body>"
                "<p>First paragraph.</p>"
                '<fig id="sa4fig1">'
                "<label>Author response image 1.</label>"
                '<graphic mimetype="image" mime-subtype="png" '
                'xlink:href="elife-70493-sa4-fig1.png" />'
                "</fig>"
                "<p>Another paragraph with an inline graphic "
                '<inline-graphic xlink:href="elife-70493-inf4.jpg" />'
                "</p>"
                "</body>"
            )
            in rough_xml_string.decode("utf8")
        )

    def test_multiple_caption(self):
        "test if there are multiple caption p tags before the inline-graphic tag"
        # register XML namespaces
        xmlio.register_xmlns()
        body_xml_string = (
            b"<body>"
            b"<p>First paragraph.</p>"
            b"<p><bold>Author response image 1.</bold></p>"
            b"<p>Caption paragraph one.</p>"
            b"<p>Caption paragraph two.</p>"
            b'<p> <inline-graphic xlink:href="elife-70493-inf3.png" /> </p>'
            b"<p>Another paragraph with an inline graphic "
            b'<inline-graphic xlink:href="elife-70493-inf4.jpg" /></p>'
            b"</body>"
        )
        xml_string = sa4_sub_article_xml_fixture(body_xml_string)

        sub_article_root = ElementTree.fromstring(xml_string)
        root = fig.transform_fig(sub_article_root, self.identifier)

        rough_xml_string = ElementTree.tostring(root, "utf-8")
        self.assertEqual(xml_string, rough_xml_string)

    def test_non_p_tag(self):
        "test if the p tag order is disrupted with a sec tag"
        # register XML namespaces
        xmlio.register_xmlns()

        body_xml_string = (
            b"<body>"
            b"<p>First paragraph.</p>"
            b"<p><bold>Author response image 1.</bold></p>"
            b"<sec>Non-p tag</sec>"
            b'<p> <inline-graphic xlink:href="elife-70493-inf3.png" /> </p>'
            b"<p>Another paragraph with an inline graphic "
            b'<inline-graphic xlink:href="elife-70493-inf4.jpg" /></p>'
            b"</body>"
        )
        xml_string = sa4_sub_article_xml_fixture(body_xml_string)

        sub_article_root = ElementTree.fromstring(xml_string)
        root = fig.transform_fig(sub_article_root, self.identifier)

        rough_xml_string = ElementTree.tostring(root, "utf-8")
        self.assertEqual(xml_string, rough_xml_string)
